<?php

namespace loyalsoft;

/**
 * 任务模块
 * @version
 *          1.0.0 Created at 2017-11-3. by --gwang
 * @author gwang (mail@wanggangzero.cn)
 * @copyright ? 2017-11-3, SJZ LoyalSoft Corporation & gwang. All rights reserved.
 */
class TaskProc {

    static function procMain($req) {
        switch ($req->cmd) {
            case CmdCode::cmd_task_getInfo:                                     # 6201 刷新任务状态
                return TaskProc::GetTaskInfo($req);
            case CmdCode::cmd_task_getReward:                                   # 6202 领取任务奖励
                return TaskProc::GetTaskReward($req);
            case CmdCode::cmd_Task_setAttentionTask:                            # 6203 设置关注任务
                return TaskProc::setAttentionTask($req);
            default:
                return Err(ErrCode::cmd_err);
        }
    }

    /**
     * [6203] 设置关注任务
     * @param req $req
     */
    static function setAttentionTask($req) {
        list($taskId) = $req->paras;
        $userTask = $req->userInfo->game->task;

        my_Assert(StlUtil::dictHasProperty($userTask->taskListPlot, $taskId), ErrCode::task_no); # 任务存在
        $taskItem = new TaskInfo($userTask->taskListPlot->$taskId);
        my_Assert($taskItem->rewardGeted < 1, ErrCode::task_reward_geted);      # 任务奖励尚未领取
        $userTask->curTaskId = $taskId;
        var_dump($userTask);
        UserProc::updateUserInfo();
        return Resp::ok(array("task" => $req->userInfo->game->task));
    }

    /**
     * [6201]更新任务状态
     * @param req $req
     */
    static function GetTaskInfo($req) {
        // 刷新
        // 返回
        return Resp::ok(array("task" => $req->userInfo->game->task));
    }

    /**
     * [6202]领取任务奖励 问:每日任务奖励和剧情任务奖励领取接口是否为一个?
     * @param req $req
     */
    static function GetTaskReward($req) {
        list($taskId) = $req->paras;                                            # 提取参数: 任务id
        $userTask = new UserTaskInfo($req->userInfo->game->task);
        # 根据任务类型分支不同任务容器 暂时未添加每日任务分支
//        var_dump($userTask->taskListPlot);
//        var_dump($req->userInfo->game->task);
        my_Assert(StlUtil::dictHasProperty($userTask->taskListPlot, $taskId), ErrCode::task_no); #判断是否存在指定任务
        $task = new TaskInfo($userTask->taskListPlot->$taskId);
        my_Assert($task->progress >= 1, ErrCode::task_progress_not_complete);   # 判断任务进度是否已完成>=1f
        my_Assert($task->rewardGeted < 1, ErrCode::task_reward_geted);          # 奖励不可重复领取
        $taskCfg = GameConfig::task_getItem($taskId);                           # 任务配置数据
        my_Assert($taskCfg != null, ErrCode::err_const_no);
        $ok = StoreProc::AddMultiItemInStore($req, $taskCfg->reward);           # 发放任务奖励
        my_Assert($ok == ErrCode::ok, $ok);
        $task->rewardGeted = 1;
        $userTask->taskListPlot->$taskId = $task;
        if ($userTask->IsAllPlotTaskOver()) {
            $userTask->curTaskNode += 1;
            $userTask->taskListPlot = ObjectInit();
            $userTask->initialize();
        }
        $req->userInfo->game->task = $userTask;                                 # 回存
//        var_dump($userTask);
        UserProc::updateUserInfo();
        return Resp::ok(array(//                                                # 返回
                    "gold" => $req->userInfo->game->baseInfo->gold,
                    "cash" => $req->userInfo->game->baseInfo->cash,
                    "tili" => $req->userInfo->game->baseInfo->tili,
                    "store" => $req->userInfo->game->store,
                    "task" => $req->userInfo->game->task
        ));
    }

// 每日任务, 自动刷新
// 
// 
    // <editor-fold defaultstate="collapsed" desc="辅助方法-比对判断">

    /**
     * 检查任务条件是否达成
     * @param TaskParams $taskParam Description
     */
    static function CheckTaskConditions($taskParam) {
        var_dump($taskParam);
        $bUpdate = false;
        foreach (req()->userInfo->game->task->taskListPlot as $tid => $task) {
            $taskCfg = GameConfig::task_getItem($tid);
            my_Assert($taskCfg != null, ErrCode::err_const_no);                 # 获取任务配置数据   
            if ($taskCfg->tasktype != $taskParam->taskType) {                   # 任务类型必须匹配
                continue;
            }
            isEditor() and $task = new TaskInfo(-1);
            if ($task->progress == 1) {                                         # 已完成的不再更新处理
                continue;
            }
            if (is_null($taskCfg->canshu1)) {
                $task->progress = 1;
                $bUpdate = true;
                continue;
            }

            if ($taskCfg->canshu1 == $taskParam->canshu1) {
                $task->tag1 = $taskParam->canshu1;
                if (null == $taskCfg->canshu2 || 0 == $taskCfg->canshu2) {
                    $task->progress = 1;
                    $task->tag2 = $taskParam->canshu2;
                    $bUpdate = true;
                } else {
                    if (is_numeric($taskCfg->canshu2)) {
                        $task->tag2 = max($task->tag2, $taskParam->canshu2);
                        if ($taskCfg->canshu2 <= $taskParam->canshu2) {
                            $bUpdate = true;
                            $task->progress = 1;
                        }
                    } else {
                        $task->tag2 = $taskParam->canshu2;
                        if ($taskCfg->canshu2 == $taskParam->canshu2) {
                            $bUpdate = true;
                            $task->progress = 1;
                        }
                    }
                }
            }
        }

        // meiri
        if ($bUpdate) {
            Resp::addTag("isTaskUpdated", true);                                # 附加tag信息, 任务有更新
        }
    }

// </editor-fold>
// 
    // <editor-fold defaultstate="collapsed" desc="处理各种任务检测事务">
    // 

    /**
     * 例如当指挥官等级提升的时候需要检查是否有任务达成
     */
    static function OnUserLevelUp($newLvl) {
        $taskParam = new TaskParams(EnumTaskType::CommanderLevelUpTo, $newLvl);
        self::CheckTaskConditions($taskParam);
    }

    static function OnHeroLevelUp($heroMoId, $newLvl) {
        $taskParam = new TaskParams(EnumTaskType::HeroLevelUpTo, $heroMoId, $newLvl);
        self::CheckTaskConditions($taskParam);
    }

    static function OnHeroGradeUp($heroMoId, $newGrade) {
        $taskParam = new TaskParams(EnumTaskType::HeroGradeUpTo, $heroMoId, $newGrade);
        self::CheckTaskConditions($taskParam);
    }

    static function OnHeroSegmengNum($heroMoId, $num) {
        $taskParam = new TaskParams(EnumTaskType::HeroSegmentNumberTo, $heroMoId, $num);
        self::CheckTaskConditions($taskParam);
    }

    static function OnHeroUnlocSkill($heroMoId, $skillId) {
        $taskParam = new TaskParams(EnumTaskType::HeroSkillUnlock, $heroMoId, $skillId);
        self::CheckTaskConditions($taskParam);
    }

    static function OnPassGateN($gateId, $difficulty) {
        $taskParam = new TaskParams(EnumTaskType::PassGateN, $gateId, $difficulty);
        self::CheckTaskConditions($taskParam);
    }

    static function OnPassCarboN($gateId, $difficulty) {
        $taskParam = new TaskParams(EnumTaskType::passCarbonN, $gateId, $difficulty);
        self::CheckTaskConditions($taskParam);
    }

    static function OnPvPWinN($num) {
        $taskParam = new TaskParams(EnumTaskType::PvPWinNumberTo, $num);
        self::CheckTaskConditions($taskParam);
    }

    /**
     * 
     * @param type $score
     */
    static function OnPvPScoreN($score) {
        $taskParam = new TaskParams(EnumTaskType::PvPScoreTo, $score);
        self::CheckTaskConditions($taskParam);
    }

    /**
     * 玩家总战力值
     * @param type $num
     */
    static function OnUserFightPowerN($num) {
        $taskParam = new TaskParams(EnumTaskType::UserPowerTo, $num);
        self::CheckTaskConditions($taskParam);
    }

    /**
     * 玩家收集积分
     * @param type $num
     */
    static function OnUserCollectScoreN($num) {
        $taskParam = new TaskParams(EnumTaskType::UserCollectScoreTo, $num);
        self::CheckTaskConditions($taskParam);
    }

    // 
// </editor-fold>
    //
}
